==> UDFclient.0.3.3.tgz: UDFclient.0.3.3/udf_discop.h <==
/* $NetBSD$ */

/*
 * File "udf_discop.h" is part of the UDFclient toolkit.
 * File $Id: udf_discop.h,v 1.16 2005/01/11 20:51:36 imago Exp $ $Name:  $
 *
 * Copyright (c) 2003, 2004 Reinoud Zandijk <reinoud@netbsd.org>
 * All rights reserved.
 *
 * The UDFclient toolkit is distributed under the Clarified Artistic Licence.
 * A copy of the licence is included in the distribution as
 * `LICENCE.clearified.artistic' and a copy of the licence can also be
 * requested at the GNU foundantion's website.
 *
 * Visit the UDFclient toolkit homepage http://www.13thmonkey.org/udftoolkit/
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


#ifndef _UDF_DISCOP_H_
#define _UDF_DISCOP_H_


#include <pthread.h>
#include "udf.h"


/* Implementation constants */
#define MAX_SESSIONS		 100
#define UDF_SECTOR_SIZE		2048
#define CD_SECTOR_LENGTH	2048
#define DISC_SECTOR_LENGTH	 512


/* Quirks */
#define CD_SESS_QUIRK_SESSION_LOCAL	1


/* State and driver constants */
#define DISC_STATE_EMPTY		0	/* nothing on the disc	 */
#define DISC_STATE_INCOMPLETE		1	/* can be appended	 */
#define DISC_STATE_FULL			2	/* no appending possible */
#define DISC_STATE_NOT_SERIAL		3	/* not serial		 */

#define SESSION_STATE_EMPTY		0	/* still empty		 */
#define SESSION_STATE_INCOMPLETE	1	/* session is open	 */
#define SESSION_STATE_CLOSED		2	/* we need a new session */
#define SESSION_STATE_COMPLETE		3	/* no appending possible */

#define UDF_DEVDRV_CLASS		0xF00
#define UDF_DEVDRV_CLASS_FILE		0x000
#define UDF_DEVDRV_CLASS_DISC		0x100
#define UDF_DEVDRV_CLASS_CD		0x200
#define UDF_DEVDRV_CLASS_DVD		0x400
/*					  C	- class
 *					   R	- recordable
 *					    W	- rewritable	*/
#define UDF_DEVDRV_TYPE_FILE		0x011
#define UDF_DEVDRV_TYPE_DISC		0x111
#define UDF_DEVDRV_TYPE_CD		0x200
#define UDF_DEVDRV_TYPE_CD_R		0x210
#define UDF_DEVDRV_TYPE_CD_RW		0x211
#define UDF_DEVDRV_TYPE_DVD		0x400
#define UDF_DEVDRV_TYPE_DVD_PLUS_R	0x410
#define UDF_DEVDRV_TYPE_DVD_PLUS_RW	0x411


/* will go? */
struct udf_session;


/*
 * The complete disc, all cd sessions and how to access the drive
 */
struct udf_discinfo {
	char	*dev_name;			/* my device name				*/
	int	 fhandle;			/* device file descriptor			*/
	void	*devhandle;			/* when file descriptors are not enough (CAM)	*/

	int	 mmc_profile;			/* in case of SCSI */
	int	 devdrv_class;

	int	 sequential;			/* media is sequential writable only		*/
	int	 recordable;			/* media is record-able; i.e. not static	*/
	int	 rewritable;			/* media can be erased selectively		*/
	int	 sector_size;			/* sector size in bytes				*/
	int	 link_size;			/* link size lossage when having underruns	*/
	int	 disc_state;			/* empty - incomplete - full   - not_serial	*/
	int	 last_session_state;		/* empty - incomplete - closed - complete	*/

	/* recording */
	int	 udf_recording;			/* flags if a recording is initialised		*/

	/* thread support & associated status flags */
	pthread_t	 rwthread_id;
	pthread_mutex_t  rwthread_lock;		/* lock for read/write thread			*/
	pthread_cond_t	 rwthread_signal;	/* signal there is work to be done		*/
	int		 rwthread_kicked;	/* sanity for spurious wakeups			*/

	pthread_mutex_t	 processed_lock;	/* lock for main threads to wait on		*/
	pthread_cond_t	 processed_signal;	/* signals an action has been done		*/	/* XXX one for all waiting threads & actions XXX */

	int		 thread_active;
	int		 finish_rw_thread;	/* ask thread to stop what its doing and exit	*/

	/* statistics */
	int		 am_writing;
	uint32_t	 sectors_read;
	uint32_t	 sectors_written;
	uint32_t	 switchings;

	/* sessions */
	int	 num_sessions;
	int	 num_udf_sessions;
	int	 session_is_UDF[MAX_SESSIONS];
	int	 session_quirks[MAX_SESSIONS];

	off_t	 session_start [MAX_SESSIONS];
	off_t	 session_end   [MAX_SESSIONS];

	uint32_t next_writable [MAX_SESSIONS];
	uint32_t free_blocks   [MAX_SESSIONS];
	uint32_t packet_size   [MAX_SESSIONS];

	STAILQ_HEAD(udf_sessions, udf_session) sessions;

	SLIST_ENTRY(udf_discinfo) next_disc;
};


/* exported functions */
extern int  udf_open_disc(char *dev_name, struct udf_discinfo **discptr);
extern int  udf_close_disc(struct udf_discinfo *disc);

extern int  udf_discinfo_get_type(struct udf_discinfo *disc);
extern int  udf_discinfo_is_cd_or_dvd(struct udf_discinfo *disc);
extern int  udf_discinfo_set_recording_parameters(struct udf_discinfo *discinfo, int testwriting);
extern int  udf_discinfo_finish_writing(struct udf_discinfo *discinfo);
extern int  udf_discinfo_reserve_track_in_logic_units(struct udf_discinfo *discinfo, uint32_t logic_units);
extern int  udf_discinfo_close_track(struct udf_discinfo *discinfo, uint16_t trackno);
extern int  udf_discinfo_repair_track(struct udf_discinfo *discinfo, uint16_t trackno);

extern int  udf_read_physical_sectors(struct udf_discinfo *disc, off_t sector, uint32_t num_sectors, char *what, uint8_t *result);
extern int  udf_write_physical_sectors(struct udf_discinfo *disc, off_t sector, uint32_t num_sectors, char *what, uint8_t *source);


#endif	/* _UDF_DISCOP_H_ */

==> UDFclient.0.4.9.tgz: UDFclient.0.4.9/udf_discop.h <==
/* $NetBSD$ */

/*
 * File "udf_discop.h" is part of the UDFclient toolkit.
 * File $Id: udf_discop.h,v 1.20 2005/06/14 23:13:49 imago Exp $ $Name:  $
 *
 * Copyright (c) 2003, 2004, 2005 Reinoud Zandijk <reinoud@netbsd.org>
 * All rights reserved.
 *
 * The UDFclient toolkit is distributed under the Clarified Artistic Licence.
 * A copy of the licence is included in the distribution as
 * `LICENCE.clearified.artistic' and a copy of the licence can also be
 * requested at the GNU foundantion's website.
 *
 * Visit the UDFclient toolkit homepage http://www.13thmonkey.org/udftoolkit/
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


#ifndef _UDF_DISCOP_H_
#define _UDF_DISCOP_H_


#include <pthread.h>
#include "udf.h"


/* Implementation constants */
#define MAX_SESSIONS		 100
#define CD_SECTOR_SIZE		2048
#define DISC_SECTOR_SIZE	 512


/* Quirks */
#define CD_SESS_QUIRK_SESSION_LOCAL	1


/* State and driver constants */
#define DISC_STATE_EMPTY		0	/* nothing on the disc	 */
#define DISC_STATE_INCOMPLETE		1	/* can be appended	 */
#define DISC_STATE_FULL			2	/* no appending possible */
#define DISC_STATE_NOT_SERIAL		3	/* not serial		 */

#define SESSION_STATE_EMPTY		0	/* still empty		 */
#define SESSION_STATE_INCOMPLETE	1	/* session is open	 */
#define SESSION_STATE_CLOSED		2	/* we need a new session */
#define SESSION_STATE_COMPLETE		3	/* no appending possible */

#define UDF_DEVDRV_CLASS		0xFF00
#define UDF_DEVDRV_CLASS_FILE		0x0000
#define UDF_DEVDRV_CLASS_DISC		0x0100
#define UDF_DEVDRV_CLASS_CD		0x0200
#define UDF_DEVDRV_CLASS_DVD		0x0400
#define UDF_DEVDRV_CLASS_MO		0x0800
#define UDF_DEVDRV_CLASS_BR		0x1000
/*					  CC	- class
 *					    R	- recordable
 *					     W	- rewritable	*/
#define UDF_DEVDRV_TYPE_FILE		0x0011
#define UDF_DEVDRV_TYPE_DISC		0x0111
#define UDF_DEVDRV_TYPE_CD		0x0200
#define UDF_DEVDRV_TYPE_CD_R		0x0210
#define UDF_DEVDRV_TYPE_CD_RW		0x0211
#define UDF_DEVDRV_TYPE_DVD		0x0400
#define UDF_DEVDRV_TYPE_DVD_R		0x0410
#define UDF_DEVDRV_TYPE_DVD_RW		0x0411
#define UDF_DEVDRV_TYPE_MO		0x0800
#define UDF_DEVDRV_TYPE_MO_R		0x0810
#define UDF_DEVDRV_TYPE_MO_RW		0x0811
#define UDF_DEVDRV_TYPE_BD		0x1000
#define UDF_DEVDRV_TYPE_BD_R		0x1010
#define UDF_DEVDRV_TYPE_BD_RE		0x1011


/* will go? */
struct udf_session;


/*
 * The complete disc, all cd sessions and how to access the drive
 */
struct udf_discinfo {
	char	*dev_name;			/* my device name				*/
	int	 fhandle;			/* device file descriptor			*/
	void	*devhandle;			/* when file descriptors are not enough (CAM)	*/

	int	 mmc_profile;			/* in case of SCSI */
	int	 devdrv_class;

	int	 sequential;			/* media is sequential writable only		*/
	int	 recordable;			/* media is record-able; i.e. not static	*/
	int	 erasable;			/* media can be erased selectively		*/
	int	 rewritable;			/* media can be rewritten			*/
	int	 packet_wr;			/* media is using packet recording		*/
	int	 sector_size;			/* sector size in bytes				*/
	int	 link_size;			/* link size lossage when having underruns	*/
	int	 disc_state;			/* empty - incomplete - full   - not_serial	*/
	int	 last_session_state;		/* empty - incomplete - closed - complete	*/

	/* recording */
	int	 udf_recording;			/* flags if a recording is initialised		*/

	/* statistics */
	int		 am_writing;
	uint32_t	 sectors_read;
	uint32_t	 sectors_written;
	uint32_t	 switchings;

	/* sessions */
	int	 num_sessions;
	int	 num_udf_sessions;
	int	 session_is_UDF[MAX_SESSIONS];
	int	 session_quirks[MAX_SESSIONS];

	off_t	 session_start [MAX_SESSIONS];
	off_t	 session_end   [MAX_SESSIONS];

	uint32_t next_writable [MAX_SESSIONS];
	uint32_t free_blocks   [MAX_SESSIONS];
	uint32_t packet_size   [MAX_SESSIONS];

	STAILQ_HEAD(udf_sessions, udf_session) sessions;

	SLIST_ENTRY(udf_discinfo) next_disc;
};


/* exported functions */
extern int  udf_open_disc(char *dev_name, struct udf_discinfo **discptr);
extern int  udf_close_disc(struct udf_discinfo *disc);

extern int  udf_discinfo_get_type(struct udf_discinfo *disc);
extern int  udf_discinfo_check_disc_ready(struct udf_discinfo *disc);
extern int  udf_discinfo_is_cd_or_dvd(struct udf_discinfo *disc);
extern int  udf_discinfo_set_recording_parameters(struct udf_discinfo *discinfo, int testwriting);
extern int  udf_discinfo_alter_perception(struct udf_discinfo *disc, uint32_t sec_size, uint32_t num_sectors);
extern int  udf_discinfo_finish_writing(struct udf_discinfo *discinfo);
extern int  udf_discinfo_reserve_track_in_logic_units(struct udf_discinfo *discinfo, uint32_t logic_units);
extern int  udf_discinfo_close_track(struct udf_discinfo *discinfo, uint16_t trackno);
extern int  udf_discinfo_repair_track(struct udf_discinfo *discinfo, uint16_t trackno);

extern int  udf_read_physical_sectors(struct udf_discinfo *disc, off_t sector, uint32_t num_sectors, char *what, uint8_t *result);
extern int  udf_write_physical_sectors(struct udf_discinfo *disc, off_t sector, uint32_t num_sectors, char *what, uint8_t *source);

/* exported helper functions */
extern int read_cd_hex2(int val);
extern int read_cd_bcd(int val);
extern int32_t cd_msf2lba(int h, int m, int s, int f);
extern int  scsi_call(int flags, struct udf_discinfo *disc, void *cmd, int cmdlen, void *data, int datalen);

#endif	/* _UDF_DISCOP_H_ */
